;**********************************************************************
;   This file is a basic code template for assembly code generation   *
;   on the PIC16F676. This file contains the basic code               *
;   building blocks to build upon.                                    *
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler (Document DS33014).                     *
;                                                                     *
;   Refer to the respective PIC data sheet for additional             *
;   information on the instruction set.                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	    ZU676.asm                                           *
;    Date:                                                            *
;    File Version:   v1                                                 *
;                                                                     *
;    Author:       NickF                                                   *
;    Company:                                                         *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files Required: P16F676.INC                                      *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;                                                                     *
;**********************************************************************

	list      p=16f676           ; list directive to define processor
	#include <p16F676.inc>        ; processor specific variable definitions

	errorlevel  -302              ; suppress message 302 from list file

	__CONFIG   _CP_OFF & _CPD_OFF & _BODEN & _MCLRE_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT 

; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.




;***** VARIABLE DEFINITIONS

	CBLOCK 0x20			;      

TenK,Thou,Hund,Tens,Ones,flags
counter,t_zn,dly,znak,znk
byH,byL,ACC1,ACC0,TEMP1,TEMP0						;byH,L-HEX , d(n) - DEC 
NumH,NumL
		ENDC


;counter -    
;flags -  4,  
;flags -  3-  
;flags -  2- 1-  , 0-  
;dly -    /  "delay"
;znak - 7-  , znk -  1,2  3 
;t_zn -  

;NumH   ;A3*16+A2
;NumL    ;A1*16+A0
;share variables
;=B4*104 + B3*103 + B2*102 + B1*101 + B0*100
;=B4*10000 + B3*1000 + B2*100 + B1*10 + B0
;TenK     ;B4
;Thou     ;B3
;Hund     ;B2
;Tens     ;B1
;Ones		B0

;**********************************************************************
		ORG     0x000             ; processor reset vector
		goto    main              ; go to beginning of program
	

		ORG     0x004             ; interrupt vector location
		goto	interr
		org	0x10
segment		;   DEC ->  .
    CLRF   PCLATH
	ADDWF  PCL, F 
	;dt	07E, 00C, 0B6, 09E, 0CC, 0DA, 0FA, 00E, 0FE, 0DE 
;-   -   
	dt	0FC, 060, 0DA, 0F2, 066, 0B6, 0BE, 0E0, 0FE, 0F6
;-   -  
;*******************************************************************
main
		; init PORTA

		bcf 	STATUS,RP0 			;Bank 0
		clrf 	PORTA 				;Clear PORTA
		movlw 	07h 				;Set RA<2:0> to 
		movwf 	CMCON 				;comp-off digital I/O
		bsf 	STATUS,RP0 			;Bank 1
		clrf 	ANSEL 				;digital I/O
		movlw 	03h 				;
		movwf 	ANSEL 				;Set RA<1:0> to analog input
		movwf 	TRISA				;Set RA<5,4,2> as output
		 				;
		clrf	IOCA				;clear Interrupt-on-Change PORTA
		bsf 	IOCA,3			;    RA3	
		clrf 	WPUA 				;  
		clrf 	PIE1 				;   		
;		movlw 	03h 				;Set POR and BOR 
;		movwf 	PCON 				;  Vdd
		bcf 	STATUS,RP0 			;Bank 0
		bsf		INTCON,PEIE			;   
		bsf		INTCON,RAIE			;    RA3
		;init PORTC

		clrf 	PORTC 				;Init PORTC
		bsf 	STATUS,RP0 			;Bank 1
		clrf 	TRISC 				;set PORTC as outputs
		bcf 	STATUS,RP0 			;Bank 0

; these first 4 instructions are not required if the internal oscillator is not use

		call    0x3FF             ; retrieve factory calibration value
		bsf     STATUS,RP0        ; set file register bank to 1 
		movwf   OSCCAL            ; update register with factory cal value 
		bcf     STATUS,RP0        ; set file register bank to 0
; ADC
	banksel		ADCON1			
		movlw	030h			;    
		movwf 	ADCON1			;RC  ~ 8 us.
	
	banksel		PIE1			;     
		bsf		PIE1, ADIE 		;   
		movlw	085h			; - 
	banksel		OPTION_REG		;   1/64
		movwf	OPTION_REG
	banksel		INTCON	
		bsf		INTCON,GIE		; 
; remaining code goes here
start
;  020h-02eh
		movlw	020h			;   
		movwf	FSR
	
c_ind	clrf	INDF			;      FSR
		incf	FSR,f			; FSR
		btfss	FSR,4			;FSR  30h? 
		goto	c_ind			;,     

;	
start_adc
		banksel	ADCON0			;   ADCON0
		btfsc	flags,2			; 
		goto	tok				;,   tok
		movlw	081h			;  
		movwf 	ADCON0			;-AN0, Vref = VDD.
		goto	zap_adc
tok		movlw	085h			;  
		movwf 	ADCON0			;-AN1, Vref = VDD.
zap_adc		bsf		ADCON0,GO		; ADC		
		movlw	6
		call	delay

		call	mult
		call	HDF
		movlw	6
		call	delay


		movlw	1
		movwf	znk				; 1  
		
st3		movlw	0e3h			
		andwf	PORTC,f			; 
		movlw	020h
		addwf	znk,w
		movwf	FSR
		movf	INDF,w
v_S		call	segment
		call	write_zn
		call	button

		incf	znk,1
		movlw	4
		subwf	znk,0
		btfss	STATUS,Z
		goto	st3
		goto	start_adc

;  
delay	movwf	dly		
del		decfsz	dly,f
		goto	del
		return

;/     LED 
;  /   znak 
;7-      
write_zn
		banksel	PORTA
		movwf	znak		;akk->znak
		clrf	PORTC		; 
		bcf		PORTA,2		;   ""
		nop					; 
		nop					; 
		nop					;---
		bsf		PORTA,2		;""    
		movlw	8
		movwf	counter		;   	
per		clrc				;   ""
		rrf		znak,f		; 
		btfss	STATUS,C	;    
		goto	wz0
		bsf		PORTC,1		;   PORTC,1
		goto	wz1
wz0		bcf		PORTC,1		;   PORTC,1
wz1		bsf		PORTC,0		; 
		nop					;   
		nop					; 
		bcf		PORTC,0		; 							
		DECFSZ	counter,f	;  
		goto	per	
		nop
		movf	znk,w
		movwf	counter
		movlw	02h
		movwf	t_zn
lwz		bcf		STATUS,C
		rlf		t_zn,f
		decfsz	counter,f
		goto	lwz
		movf	t_zn,w		
		iorwf	PORTC,f			; 
		call	pause

		return
pause
		movlw	0ffh			;
		call	delay
		call	delay
		call	delay
		call	delay
		call	delay
		call	delay
		call	delay
		call	delay
		call	delay
		call	delay
		return
;/  
button
	banksel		PORTA
		BTFSC	flags,3
		goto	bt2			;
		bcf		PORTA,4		; B2  
		goto	bt3
bt2		bsf		PORTA,4
bt3		BTFSC	flags,4
		goto	bt5	
		bcf		PORTA,5		; B3  
		goto	bt6
bt5		bsf		PORTA,5
bt6		return

;/  
interr
		movwf	05fh
	banksel		INTCON	
		bcf		INTCON,GIE			;  
		btfss	PIR1,ADIF			;  
		goto	int_ra3				;   RA3
int_ADC
		bcf		PIR1,ADIF			;  ADIF
	banksel		ADCON0
		bcf		ADCON0,ADON			; ADC
	banksel		ADRESL
		movf	ADRESL,w
		movwf	byL					; . 
	banksel		ADRESH
		movf	ADRESH,w
		movwf	byH					; . 
		goto	isr
int_ra3								
		btfsc	PORTA,3				;   	
		goto	it					;   ,   
		movf	PORTC,w				;  
		andlw	01ch				;  "flags"  
		iorwf	flags,0				;   (7,6,5:1,0)
		goto	isr
it		; 					; 
isr
	banksel		INTCON
		bcf		INTCON,RAIF			;  RAIF
		bsf		INTCON,GIE			; 
		movf	05fh, w
		retfie                    	; return from interrupt
;/    "input" / 
mult
		movf	byH,w
		movwf	ACC1
		movf	byL,w
		movwf	ACC0
;/  16-    (4.887d)
multik
; ACC = ACC * 4.887
; Temp = TEMP
; ACC size = 10 bits
; Error = 0.5 %
; Bytes order = little endian
; Round = no

; ALGORITHM:
; Clear accumulator
; Add input * 4 to accumulator
; Add input * 1 to accumulator
; Substract input / 8 from accumulator
; Move accumulator to result
;
; Approximated constant: 4.875, Error: 0.245549 %

;     Input: ACC0 .. ACC1, 10 bits
;    Output: ACC0 .. ACC1, 13 bits
; Code size: 33 instructions

;copy accumulator to temporary
	movf	ACC1, w
	movwf	TEMP1
	movf	ACC0, w
	movwf	TEMP0


;shift accumulator right 3 times
	clrc
	rrf	ACC1, f
	rrf	ACC0, f
	clrc
	rrf	ACC1, f
	rrf	ACC0, f
	clrc
	rrf	ACC0, f

;negate accumulator
	comf	ACC0, f
	incf	ACC0, f

;add temporary to accumulator
	clrf	ACC1
	btfsc	ACC0, 7
	comf	ACC1, f
	addwf	ACC0, f
	movf	TEMP1, w
	skpnc
	incfsz	TEMP1, w
	addwf	ACC1, f

;shift temporary left 2 times
	clrc
	rlf	TEMP0, f
	rlf	TEMP1, f
	rlf	TEMP0, f
	rlf	TEMP1, f

;add temporary to accumulator
	movf	TEMP0, w
	addwf	ACC0, f
	movf	TEMP1, w
	skpnc
	incfsz	TEMP1, w
	addwf	ACC1, f
	return

  ; Ideal for displaying A/D values
  
;Takes hex number in NumH:NumL  Returns decimal in ;TenK:Thou:Hund:Tens:Ones
;written by John Payson

;input
;=A3*163 + A2*162 + A1*161 + A0*160
;=A3*4096 + A2*256 + A1*16 + A0
;NumH            EQU AD3M       ;A3*16+A2
;NumL            EQU AD3L	;A1*16+A0
;share variables
;=B4*104 + B3*103 + B2*102 + B1*101 + B0*100
;=B4*10000 + B3*1000 + B2*100 + B1*10 + B0
;TenK            EQU LOOPER      ;B4
;Thou            EQU D2		;B3
;Hund            EQU D1		;B2
;Tens            EQU R2		;B1
;Ones            EQU R1		;B0

HDF			
			movf	ACC1,w
			movwf	NumH
			movf	ACC0,w
			movwf	NumL
HexDecFast
		swapf	NumH,w	;w  = A2*16+A3
        andlw   0x0F     ;w  = A3		*** PERSONALLY, I'D REPLACE THESE 2
        addlw   0xF0	;w  = A3-16	*** LINES WITH "IORLW b'11110000B' " -AW
        movwf   Thou	;B3 = A3-16
        addwf   Thou,f	;B3 = 2*(A3-16) = 2A3 - 32
        addlw   .226	;w  = A3-16 - 30 = A3-46
        movwf   Hund	;B2 = A3-46
        addlw   .50	;w  = A3-46 + 50 = A3+4
        movwf   Ones	;B0 = A3+4

        movf    NumH,w	;w  = A3*16+A2
        andlw   0x0F	;w  = A2
        addwf   Hund,f	;B2 = A3-46 + A2 = A3+A2-46
        addwf   Hund,f	;B2 = A3+A2-46  + A2 = A3+2A2-46
        addwf   Ones,f	;B0 = A3+4 + A2 = A3+A2+4
        addlw   .233	;w  = A2 - 23
        movwf   Tens	;B1 = A2-23
        addwf   Tens,f	;B1 = 2*(A2-23)
        addwf   Tens,f	;B1 = 3*(A2-23) = 3A2-69 (Doh! thanks NG)

        swapf   NumL,w	;w  = A0*16+A1
        andlw   0x0F	;w  = A1
        addwf   Tens,f	;B1 = 3A2-69 + A1 = 3A2+A1-69 range -69...-9
        addwf   Ones,f	;B0 = A3+A2+4 + A1 = A3+A2+A1+4 and Carry = 0 (thanks NG)

        rlf     Tens,f	;B1 = 2*(3A2+A1-69) + C = 6A2+2A1-138 and Carry is now 1 as tens register had to be negitive
        rlf     Ones,f	;B0 = 2*(A3+A2+A1+4) + C = 2A3+2A2+2A1+9 (+9 not +8 due to the carry from prev line, Thanks NG)
        comf    Ones,f	;B0 = ~(2A3+2A2+2A1+9) = -2A3-2A2-2A1-10 (ones complement plus 1 is twos complement. Thanks SD)
;;Nikolai Golovchenko [golovchenko at MAIL.RU] says: comf can be regarded like:
;;      comf Ones, f
;;      incf Ones, f
;;      decf Ones, f
;;First two instructions make up negation. So,
;;Ones  = -1 * Ones - 1 
;;      = - 2 * (A3 + A2 + A1) - 9 - 1 
;;      = - 2 * (A3 + A2 + A1) - 10
        rlf     Ones,f	;B0 = 2*(-2A3-2A2-2A1-10) = -4A3-4A2-4A1-20

        movf    NumL,w	;w  = A1*16+A0
        andlw   0x0F	;w  = A0
        addwf   Ones,f	;B0 = -4A3-4A2-4A1-20 + A0 = A0-4(A3+A2+A1)-20 range -215...-5 Carry=0
        rlf     Thou,f	;B3 = 2*(2A3 - 32) = 4A3 - 64

        movlw   0x07	;w  = 7
        movwf   TenK	;B4 = 7

;B0 = A0-4(A3+A2+A1)-20	;-5...-200
;B1 = 6A2+2A1-138	;-18...-138
;B2 = A3+2A2-46		;-1...-46
;B3 = 4A3-64		;-4...-64
;B4 = 7			;7
; At this point, the original number is
; equal to TenK*10000+Thou*1000+Hund*100+Tens*10+Ones 
; if those entities are regarded as two's compliment 
; binary.  To be precise, all of them are negative 
; except TenK.  Now the number needs to be normal- 
; ized, but this can all be done with simple byte 
; arithmetic.

        movlw   .10	;w  = 10
Lb1:			;do
        addwf   Ones,f	; B0 += 10
        decf    Tens,f	; B1 -= 1
        btfss   3,0
	;skip no carry
         goto   Lb1	; while B0 < 0
	;jmp carry
Lb2:			;do
        addwf   Tens,f	; B1 += 10
        decf    Hund,f	; B2 -= 1
        btfss   3,0
         goto   Lb2	; while B1 < 0
Lb3:			;do
        addwf   Hund,f	; B2 += 10
        decf    Thou,f	; B3 -= 1
        btfss   3,0
         goto   Lb3	; while B2 < 0
Lb4:			;do
        addwf   Thou,f	; B3 += 10
        decf    TenK,f	; B4 -= 1
        btfss   3,0
         goto   Lb4	; while B3 < 0

        retlw   0



;      RC 
			ORG   0x3FF 
           retlw   .128
; initialize eeprom locations
		
		ORG	0x2100
		DE	0x00, 0x01, 0x02, 0x03

		END                 ; directive 'end of program'
